'use strict'

exports.__esModule = true
exports.default = exports.useDebounce = exports.useDebounceCallback = void 0

var _react = require('react')

var _requestTimeout = require('@essentials/request-timeout')

const useDebounceCallback = (fn, wait = 100, leading = false) => {
  const timeout = (0, _react.useRef)(null)
  const next = (0, _react.useCallback)(
    (this_, args) => {
      timeout.current = null
      leading !== true && fn.apply(this_, args)
    },
    [fn]
  ) // cleans up pending timeouts when the function chagnes

  function _ref() {
    if (timeout.current !== null) {
      ;(0, _requestTimeout.clearRequestTimeout)(timeout.current)
      timeout.current = null
    }
  }

  ;(0, _react.useEffect)(() => _ref, [fn, wait])
  return (0, _react.useCallback)(
    function() {
      const this_ = this
      const args = arguments

      if (timeout.current === null && leading === true) {
        fn.apply(this_, args)
      } else if (timeout.current !== null) {
        ;(0, _requestTimeout.clearRequestTimeout)(timeout.current)
      }

      timeout.current = (0, _requestTimeout.requestTimeout)(
        () => next(this_, args),
        wait
      )
    },
    [fn, wait]
  )
}

exports.useDebounceCallback = useDebounceCallback

const useDebounce = (initialState, wait, leading) => {
  const [state, setState] = (0, _react.useState)(initialState)
  return [state, useDebounceCallback(setState, wait, leading)]
}

exports.useDebounce = useDebounce
var _default = useDebounce
exports.default = _default
